define PS2_SELF

include "ps2.s"

; Init PS2 keyboard
xdef InitPS2

InitPS2:
	;
	ld	bc,	$EFF7
	ld	a,	$80
	out	(c),a
	;
	ld	bc,	$DFF7
	ld	a,	$FF
	out	(c),a
	;
	ld	bc,	$BFF7
	ld	a,	$02
	out	(c),a
	;
	ret

; Get scancode in HL
; L - scancode
; H - flags
;
; if E0 prefix then L bit 7 is 1
; if F0 (keyup) then H bit 0 is 1

xdef GetPS2
GetPS2:

ps2kbd_loop:
	;
	ld	bc,	$DFF7
	ld	a,	$FF
	out	(c),a
	;
	ld	bc,	$BFF7
	ld	a,	$02
	out	(c),a
	;
	in	a,(c)
	;
	cp	$00
	jr	z,ps2kbd_00
	cp	$F0
	jr	z,ps2kbd_F0
	cp	$E0
	jr	z,ps2kbd_E0
	cp	$FF
	jr	z,ps2kbd_FF
	jr	ps2kbd_translate
ps2kbd_FF:
	;
	;
	ld	bc,	$DFF7
	ld	a,	$0C
	out	(c),a
	ld	bc,	$BFF7
	ld	a,	$01
	out	(c),a
	;
ps2kbd_00:
	ld	hl,0
	ret
	;
ps2kbd_F0:
	ld	a,(ps2kbd_status)
	or	$01
	ld	(ps2kbd_status),a
	jr	ps2kbd_loop
	;
ps2kbd_E0:
	ld	a,(ps2kbd_status)
	or	$80
	ld	(ps2kbd_status),a
	jr	ps2kbd_loop
	;
ps2kbd_translate:
	ld	l,a
	ld	a,(ps2kbd_status)
	ld	h,a
	; if E0 - scancode+$80
	and	$80
	or	l
	ld	l,a
	; push up
	ld	a,h
	and	$01
	ld	h,a
	;
	xor	a
	ld	(ps2kbd_status),a
	;
	ret
	;
ps2kbd_status:	defb $00

;  KEXTEND -   8
defc KEXTEND=	1
;  ICAPS -    
defc ICAPS=	2

;    L
; :
;	HL: L - scancode H - flags
; :
;	HL -   (6 )
;	B = bit 1 0-pressed, 1-unpressed
;	a==0 -   
SearchPS2:
	push	hl
	pop	bc ; bc = hl
	;
	ld	hl,recode_table
	ld	de,$0006 ;  
SearchPS2Loop:
	ld a,(hl)
	;  ?
	or a
	ret z
	;   ?
	cp c
	ret z
	;  
	add hl, de
	jr SearchPS2Loop

;        
xdef InKey
InKey:
	call	GetPS2
	call	SearchPS2
	or	a
	jr	nz,inkey_found_key
inkey_not_found_key:
	ld	hl,$0000
	ret
	;
inkey_found_key:
	push	hl
	pop	ix ; ix=hl
	;
	push	bc
	call	get_ext_code
	pop	bc
	; HL - ext code
	; B[0] = unpressed
	bit 0,b
	jr nz,inkey_unpress
	bit 0,h
	jr nz,inkey_press_ext
	;    
	ret
;   
inkey_press_ext:
	ld a,l
	; CAPS
	cp CAPS
	jr nz, noCAPS
	;
	ld a,(ModePS2_P)
	xor 2^PS2_CAPS
	ld (ModePS2_P),a
	ld	hl,$0000
	ret
noCAPS:
	; SHIFT
	ld a,l
	cp LSHIFT
	push af
	cp RSHIFT
	push af
	pop bc
	pop de
	ld a,c
	or e
	bit 6,a
	jr z,noSHIFTp
	;
	ld a,(ModePS2_T)
	set PS2_SHIFT,a
	ld (ModePS2_T),a
	ld	hl,$0000
	ret
noSHIFTp:
	; ALT
	ld a,l
	cp LALT
	push af
	cp RALT
	push af
	pop bc
	pop de
	ld a,c
	or e
	bit 6,a
	jr z,noALTp
	;
	ld a,(ModePS2_T)
	set PS2_ALT,a
	ld (ModePS2_T),a
	ld	hl,$0000
	ret
noALTp:
	; CTRL
	ld a,l
	cp LCTRL
	push af
	cp RCTRL
	push af
	pop bc
	pop de
	ld a,c
	or e
	bit 6,a
	jr z,noCTRLp
	;
	ld a,(ModePS2_T)
	set PS2_CTRL,a
	ld (ModePS2_T),a
	ld	hl,$0000
	ret
noCTRLp:
	ret

;  
inkey_unpress:
	; ,   
	bit 0,h
	jr z,inkey_not_found_key
	ld a,l
	; CAPS
	cp CAPS
	jr nz, noCAPSu
	ld	hl,$0000
	ret
noCAPSu:
	; SHIFT
	ld a,l
	cp LSHIFT
	push af
	cp RSHIFT
	push af
	pop bc
	pop de
	ld a,c
	or e
	bit 6,a
	jr z,noSHIFTu
	;
	ld a,(ModePS2_T)
	res PS2_SHIFT,a
	ld (ModePS2_T),a
	; check RUS-LAT
	bit PS2_CTRL,a
	jr z, noRusLat
	; 
	ld a,(ModePS2_P)
	xor 2^PS2_RUS
	ld (ModePS2_P),a
noRusLat:
	ld	hl,$0000
	ret
noSHIFTu:
	; ALT
	ld a,l
	cp LALT
	push af
	cp RALT
	push af
	pop bc
	pop de
	ld a,c
	or e
	bit 6,a
	jr z,noALTu
	ld a,(ModePS2_T)
	res PS2_ALT,a
	ld (ModePS2_T),a
	ld	hl,$0000
	ret
noALTu:; CTRL
	ld a,l
	cp LCTRL
	push af
	cp RCTRL
	push af
	pop bc
	pop de
	ld a,c
	or e
	bit 6,a
	jr z,noCTRLu
	ld a,(ModePS2_T)
	res PS2_CTRL,a
	ld (ModePS2_T),a
	ld	hl,$0000
	ret
noCTRLu:
	ret

;       ModePS2
; ix =  
; ix+1 - . 
; ix+2 - . 
; ix+3 - . 
; ix+4 - . 
; ix+5 - .
;	ICAPS -    CAPS ( shift)
;	KEXTEND -   () >255
;  - HL -   (l - KOI8,  h=0)
get_ext_code:
	; rus-lat
	ld	a,(ModePS2_P)
	ld	c,a
	ld	a,(ModePS2_T)
	xor	a,c ; - 
	; -
	bit	PS2_RUS,a
	jr	z, get_ext_code_lat
	ld	de,$003 ;     IX
	jr	get_ext_check_shift
	ret

get_ext_code_lat:
	ld	de,$001 ;     IX
	;  Shift  Caps
get_ext_check_shift:
	ld	c,0 ;   CAPS
	ld	a,ICAPS
	and	(ix+5)
	jr nz,	get_ext_igncaps
	;  CAPS
	ld	a,(ModePS2_P)
	ld	c,a
	;  CAPS
get_ext_igncaps:
	ld	a,(ModePS2_T)
	xor	a,c ; - 
	;
	bit	PS2_CAPS,a
	jr	z, get_ext_code_small
	;  
	inc	de
get_ext_code_small:
	;   -  HL
	ld	a,KEXTEND
	and	(ix+5)
	ld	h,a
	add	ix,de
	ld	l,(ix+0)
	;
	ret

;  
ModePS2_P: ; PS2_RUS PS2_CAPS
	defb	0x00

;  
ModePS2_T: ; PS2_ALT PS2_SHIFT PS2_CTRL
	defb	0x00
	
;   (KOI8R)
recode_table:
	;  scan	llat	ulat	lrus	urus	flags
	;  +0	+1	+2	+3	+4	+5

	; . 
	defb $1C,	'a',	'A',	'',	'',	0
	defb $32,	'b',	'B',	'',	'',	0
	defb $21,	'c',	'C',	'',	'',	0
	defb $23,	'd',	'D',	'',	'',	0
	defb $24,	'e',	'E',	'',	'',	0
	defb $2B,	'f',	'F',	'',	'',	0
	defb $34,	'g',	'G',	'',	'',	0
	defb $33,	'h',	'H',	'',	'',	0
	defb $43,	'i',	'I',	'',	'',	0
	defb $3B,	'j',	'J',	'',	'',	0
	defb $42,	'k',	'K',	'',	'',	0
	defb $4B,	'l',	'L',	'',	'',	0
	defb $3A,	'm',	'M',	'',	'',	0
	defb $31,	'n',	'N',	'',	'',	0
	defb $44,	'o',	'O',	'',	'',	0
	defb $4D,	'p',	'P',	'',	'',	0
	defb $15,	'q',	'Q',	'',	'',	0
	defb $2D,	'r',	'R',	'',	'',	0
	defb $1B,	's',	'S',	'',	'',	0
	defb $2C,	't',	'T',	'',	'',	0
	defb $3C,	'u',	'U',	'',	'',	0
	defb $2A,	'v',	'V',	'',	'',	0
	defb $1D,	'w',	'W',	'',	'',	0
	defb $22,	'x',	'X',	'',	'',	0
	defb $35,	'y',	'Y',	'',	'',	0
	defb $1A,	'z',	'Z',	'',	'',	0
	defb $54,	'[',	'{',	'',	'',	0
	defb $5B,	']',	'}',	'',	'',	0
	defb $4C,	$3B,	':',	'',	'',	0 ; -;-
	defb $52,	$27,	$22,	'',	'',	0 ; ' "
	defb $41,	',',	'<',	'',	'',	0
	defb $49,	'.',	'>',	'',	'',	0
	defb $0E,	'`',	'~',	'',	'',	0
	defb $29,	' ',	' ',	' ',	' ',	0 ; 

	; 
	defb $4A,	'/',	'?',	'.',	',',	ICAPS
	defb $5D,	'\',	'|',	'\',	'/',	ICAPS
	defb $4E,	'-',	'_',	'-',	'_',	ICAPS
	defb $55,	'=',	'+',	'=',	'+',	ICAPS

	; 
	defb $45,	'0',	$29,	'0',	$29,	ICAPS ; )
	defb $16,	'1',	'!',	'1',	'!',	ICAPS
	defb $1E,	'2',	'@',	'2',	'"',	ICAPS
	defb $26,	'3',	'#',	'3',	'N',	ICAPS
	defb $25,	'4',	'$',	'4',	$3B,	ICAPS ;    ...
	defb $2E,	'5',	'%',	'5',	'%',	ICAPS
	defb $36,	'6',	'^',	'6',	':',	ICAPS
	defb $3D,	'7',	'&',	'7',	'?',	ICAPS
	defb $3E,	'8',	'*',	'8',	'*',	ICAPS
	defb $46,	'9',	$28,	'9',	$28,	ICAPS ; (

	; 
	defb $5A,	$0A,	$0A,	$0A,	$0A,	0 ; Enter
	defb $66,	$08,	$08,	$08,	$08,	0 ; Back Space
	defb $0D,	$09,	$09,	$09,	$09,	0 ; TAB
	defb $76,	$1B,	$1B,	$1B,	$1B,	0 ; ESC
	
	; 
	defb $58,	CAPS,	CAPS,	CAPS,	CAPS,	KEXTEND
	
	defb $12, LSHIFT, LSHIFT, LSHIFT, LSHIFT, KEXTEND
	defb $59, RSHIFT, RSHIFT, RSHIFT, RSHIFT, KEXTEND

	defb $11, LALT, LALT, LALT, LALT, KEXTEND
	defb $91, RALT, RALT, RALT, RALT, KEXTEND

	defb $14, LCTRL, LCTRL, LCTRL, LCTRL, KEXTEND
	defb $94, RCTRL, RCTRL, RCTRL, RCTRL, KEXTEND

	defb $9F, LWIN, LWIN, LWIN, LWIN, KEXTEND
	defb $A7, RWIN, RWIN, RWIN, RWIN, KEXTEND

	defb $F0, INSERT, INSERT, INSERT, INSERT, KEXTEND
	defb $EC, HOME, HOME, HOME, HOME, KEXTEND

	defb $FD, PGUP, PGUP, PGUP, PGUP, KEXTEND
	defb $FA, PGDN, PGDN, PGDN, PGDN, KEXTEND

	defb $EC, KDELETE, KDELETE, KDELETE, KDELETE, KEXTEND
	defb $E9, KEND, KEND, KEND, KEND, KEXTEND

	defb $F5, UP, UP, UP, UP, KEXTEND
	defb $F2, DN, DN, DN, DN, KEXTEND
	defb $EB, LEFT, LEFT, LEFT, LEFT, KEXTEND
	defb $F4, RIGHT, RIGHT, RIGHT, RIGHT, KEXTEND

	defb $05, F1,F1,F1,F1, KEXTEND
	defb $06, F2,F2,F2,F2, KEXTEND
	defb $04, F3,F3,F3,F3, KEXTEND
	defb $0C, F4,F4,F4,F4, KEXTEND
	defb $03, F5,F5,F5,F5, KEXTEND
	defb $0B, F6,F6,F6,F6, KEXTEND
	defb $83, F7,F7,F7,F7, KEXTEND
	defb $0A ,F8,F8,F8,F8, KEXTEND
	defb $01 ,F9,F9,F9,F9, KEXTEND
	defb $09 ,F10,F10,F10,F10, KEXTEND
	defb $78 ,F11,F11,F11,F11, KEXTEND
	defb $07 ,F12,F12,F12,F12, KEXTEND

	defb $77, NUMLOCK, NUMLOCK, NUMLOCK, NUMLOCK, KEXTEND
	defb $CA, KPDIV, KPDIV, KPDIV, KPDIV, KEXTEND
	defb $7C, KPMUL, KPMUL, KPMUL, KPMUL, KEXTEND
	defb $7B, KPMIMUS, KPMIMUS, KPMIMUS, KPMIMUS, KEXTEND
	defb $79, KPPLUS, KPPLUS, KPPLUS, KPPLUS, KEXTEND
	defb $DA, KPENTER, KPENTER, KPENTER, KPENTER, KEXTEND
	defb $71, KPPOINT, KPPOINT, KPPOINT, KPPOINT, KEXTEND

	defb $70, KP0, KP0, KP0, KP0, KEXTEND
	defb $69, KP1, KP1, KP1, KP1, KEXTEND
	defb $72, KP2, KP2, KP2, KP2, KEXTEND
	defb $7A, KP3, KP3, KP3, KP3, KEXTEND
	defb $6B, KP4, KP4, KP4, KP4, KEXTEND
	defb $73, KP5, KP5, KP5, KP5, KEXTEND
	defb $74, KP6, KP6, KP6, KP6, KEXTEND
	defb $6C, KP7, KP7, KP7, KP7, KEXTEND
	defb $75, KP8, KP8, KP8, KP8, KEXTEND
	defb $7D, KP9, KP9, KP9, KP9, KEXTEND

	;  
	defb $00
